//   Vector Base class for CS4731: COMPUTER GRAPHICS
//   Copyright (C) 1999,2000  Mark R. Stevens
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the Free Software
//   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#ifndef VECTOR
#define VECTOR

// Vector class definition
class Vector {

 public:

  // constructors
//  Vector (const float x=0.0, const float y=0.0, const float z=0.0, const float w=1.0);
  Vector (const float x, const float y, const float z, const float w);

  // access data
  float   x () const;
  float   y () const;
  float   z () const;
  float   w () const;

  float & x ();
  float & y ();
  float & z ();
  float & w ();

  // methods
  float magnitude() const;
  void  normalize();
  void  set(float nx, float ny, float nz);

  // operators
  friend Vector operator + (const Vector &v1, const Vector &v2);
  friend Vector operator - (const Vector &v1, const Vector &v2);
  friend float  dot        (const Vector &v1, const Vector &v2);
  friend Vector cross      (const Vector &v1, const Vector &v2);

 private:

  // local data
  float data[4];

};

#endif





