#ifndef _hill_house_h_
#define _hill_house_h_


#include "cs4731app.h"
#ifdef __APPLE_CC__
#include "gl.h"
#else
#include "GL/gl.h"
#endif

class hill_house : public cs4731app {
public:

    // initialize
    virtual void init(minigl &mgl, miniglut &mglut);

    // redraw the screen
    virtual void draw(minigl &mgl, miniglut &mglut);

    // change the window size
    virtual void reshape(minigl &mgl, miniglut &mglut, int width, int height) {}

    // outputs the usage to the screen to show what the keys do
    virtual void usage();

    // receive keyboard events
    virtual void key    (minigl &mgl, miniglut &mglut, unsigned char k, int x, int y);

    // receive mouse events
    virtual void mouse (minigl &mgl, miniglut &mglut, int button, int state, int x, int y) {}

    // receive special keyboard events (arrow keys, etc.)
    virtual void special(minigl &mgl, miniglut &mglut, int k, int x, int y);

};

#endif
