//   enumerated types used by mesa for interface with cs4731
//   Copyright (C) 1999,2000  Mark R. Stevens
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the Free Software
//   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//   Taken From:
//      Mesa 3-D graphics library
//      Version:  3.1
//
//      Copyright (C) 1999  Brian Paul   All Rights Reserved.   

#ifndef MGLUTENUM
#define MGLUTENUM

typedef enum {

  MGLUT_KEY_LEFT =                  100,
  MGLUT_KEY_UP =                    101,
  MGLUT_KEY_RIGHT =                 102,
  MGLUT_KEY_DOWN =                  103,
  MGLUT_KEY_PAGE_UP =               104,
  MGLUT_KEY_PAGE_DOWN =             105,
  MGLUT_KEY_HOME =                  106,
  MGLUT_KEY_END =                   107,
  MGLUT_KEY_INSERT =                108,
  MGLUT_ELAPSED_TIME =              700,

  //adding these for the mouse
  MGLUT_DOWN =                      0,
  MGLUT_UP =                        1

} MGLUTenum;

#endif


