//   Vertex class for CS4731: COMPUTER GRAPHICS
//   Copyright (C) 1999,2000  Mark R. Stevens
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the Free Software
//   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

// required includes
#include "Vertex.h"


//--------------------------------------------------------------------------------  
Vertex::Vertex (float x = 0.0, 
		float y = 0.0, 
		float z = 0.0, 
		float w = 1.0) {
  p[0] = x;
  p[1] = y;
  p[2] = z;
  p[3] = w;

}  
//--------------------------------------------------------------------------------  

//--------------------------------------------------------------------------------  
float   Vertex::x () const { return(p[0]);  }
float   Vertex::y () const { return(p[1]);  }
float   Vertex::z () const { return(p[2]);  }
float   Vertex::w () const { return(p[3]);  }
float & Vertex::x ()       { return(p[0]);  }
float & Vertex::y ()       { return(p[1]);  }
float & Vertex::z ()       { return(p[2]);  }
float & Vertex::w ()       { return(p[3]);  }
//--------------------------------------------------------------------------------  

//--------------------------------------------------------------------------------  
Vertex Vertex::homogenous() const 
{
  return Vertex (p[0] / p[3], p[1] / p[3], p[2] / p[3], p[3] / p[3]);

}
//--------------------------------------------------------------------------------  






