//   enumerated types used by OpenGL for interface with cs4731
//   Copyright (C) 1999,2000  Mark R. Stevens
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the Free Software
//   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//   Taken From:
//      Mesa 3-D graphics library
//      Version:  3.1
//
//      Copyright (C) 1999  Brian Paul   All Rights Reserved.   

#ifndef MGLENUM
#define MGLENUM

typedef enum {

	/* Boolean values */
	MGL_FALSE			= 0,
	MGL_TRUE				= 1,

	/* Data types */
	MGL_BYTE				= 0x1400,
	MGL_UNSIGNED_BYTE		= 0x1401,
	MGL_SHORT			= 0x1402,
	MGL_UNSIGNED_SHORT		= 0x1403,
	MGL_INT				= 0x1404,
	MGL_UNSIGNED_INT			= 0x1405,
	MGL_FLOAT			= 0x1406,
	MGL_DOUBLE			= 0x140A,
	MGL_2_BYTES			= 0x1407,
	MGL_3_BYTES			= 0x1408,
	MGL_4_BYTES			= 0x1409,

	/* Primitives */
	MGL_POINTS			= 0x0000,
	MGL_LINES			= 0x0001,
	MGL_LINE_LOOP			= 0x0002,
	MGL_LINE_STRIP			= 0x0003,
	MGL_TRIANGLES			= 0x0004,
	MGL_TRIANGLE_STRIP		= 0x0005,
	MGL_TRIANGLE_FAN			= 0x0006,
	MGL_QUADS			= 0x0007,
	MGL_QUAD_STRIP			= 0x0008,
	MGL_POLYGON			= 0x0009,

	/* Vertex Arrays */
	MGL_VERTEX_ARRAY			= 0x8074,
	MGL_NORMAL_ARRAY			= 0x8075,
	MGL_COLOR_ARRAY			= 0x8076,
	MGL_INDEX_ARRAY			= 0x8077,
	MGL_TEXTURE_COORD_ARRAY		= 0x8078,
	MGL_EDGE_FLAG_ARRAY		= 0x8079,
	MGL_VERTEX_ARRAY_SIZE		= 0x807A,
	MGL_VERTEX_ARRAY_TYPE		= 0x807B,
	MGL_VERTEX_ARRAY_STRIDE		= 0x807C,
	MGL_NORMAL_ARRAY_TYPE		= 0x807E,
	MGL_NORMAL_ARRAY_STRIDE		= 0x807F,
	MGL_COLOR_ARRAY_SIZE		= 0x8081,
	MGL_COLOR_ARRAY_TYPE		= 0x8082,
	MGL_COLOR_ARRAY_STRIDE		= 0x8083,
	MGL_INDEX_ARRAY_TYPE		= 0x8085,
	MGL_INDEX_ARRAY_STRIDE		= 0x8086,
	MGL_TEXTURE_COORD_ARRAY_SIZE	= 0x8088,
	MGL_TEXTURE_COORD_ARRAY_TYPE	= 0x8089,
	MGL_TEXTURE_COORD_ARRAY_STRIDE	= 0x808A,
	MGL_EDGE_FLAG_ARRAY_STRIDE	= 0x808C,
	MGL_VERTEX_ARRAY_POINTER		= 0x808E,
	MGL_NORMAL_ARRAY_POINTER		= 0x808F,
	MGL_COLOR_ARRAY_POINTER		= 0x8090,
	MGL_INDEX_ARRAY_POINTER		= 0x8091,
	MGL_TEXTURE_COORD_ARRAY_POINTER	= 0x8092,
	MGL_EDGE_FLAG_ARRAY_POINTER	= 0x8093,
	MGL_V2F				= 0x2A20,
	MGL_V3F				= 0x2A21,
	MGL_C4UB_V2F			= 0x2A22,
	MGL_C4UB_V3F			= 0x2A23,
	MGL_C3F_V3F			= 0x2A24,
	MGL_N3F_V3F			= 0x2A25,
	MGL_C4F_N3F_V3F			= 0x2A26,
	MGL_T2F_V3F			= 0x2A27,
	MGL_T4F_V4F			= 0x2A28,
	MGL_T2F_C4UB_V3F			= 0x2A29,
	MGL_T2F_C3F_V3F			= 0x2A2A,
	MGL_T2F_N3F_V3F			= 0x2A2B,
	MGL_T2F_C4F_N3F_V3F		= 0x2A2C,
	MGL_T4F_C4F_N3F_V4F		= 0x2A2D,

	/* Matrix Mode */
	MGL_MATRIX_MODE			= 0x0BA0,
	MGL_MODELVIEW			= 0x1700,
	MGL_PROJECTION			= 0x1701,
	MGL_TEXTURE			= 0x1702,

	/* Points */
	MGL_POINT_SMOOTH			= 0x0B10,
	MGL_POINT_SIZE			= 0x0B11,
	MGL_POINT_SIZE_GRANULARITY 	= 0x0B13,
	MGL_POINT_SIZE_RANGE		= 0x0B12,

	/* Lines */
	MGL_LINE_SMOOTH			= 0x0B20,
	MGL_LINE_STIPPLE			= 0x0B24,
	MGL_LINE_STIPPLE_PATTERN		= 0x0B25,
	MGL_LINE_STIPPLE_REPEAT		= 0x0B26,
	MGL_LINE_WIDTH			= 0x0B21,
	MGL_LINE_WIDTH_GRANULARITY	= 0x0B23,
	MGL_LINE_WIDTH_RANGE		= 0x0B22,

	/* Polygons */
	MGL_POINT			= 0x1B00,
	MGL_LINE				= 0x1B01,
	MGL_FILL				= 0x1B02,
	MGL_CW				= 0x0900,
	MGL_CCW				= 0x0901,
	MGL_FRONT			= 0x0404,
	MGL_BACK				= 0x0405,
	MGL_POLYGON_MODE			= 0x0B40,
	MGL_POLYGON_SMOOTH		= 0x0B41,
	MGL_POLYGON_STIPPLE		= 0x0B42,
	MGL_EDGE_FLAG			= 0x0B43,
	MGL_CULL_FACE			= 0x0B44,
	MGL_CULL_FACE_MODE		= 0x0B45,
	MGL_FRONT_FACE			= 0x0B46,
	MGL_POLYGON_OFFSET_FACTOR	= 0x8038,
	MGL_POLYGON_OFFSET_UNITS		= 0x2A00,
	MGL_POLYGON_OFFSET_POINT		= 0x2A01,
	MGL_POLYGON_OFFSET_LINE		= 0x2A02,
	MGL_POLYGON_OFFSET_FILL		= 0x8037,

	/* Display Lists */
	MGL_COMPILE			= 0x1300,
	MGL_COMPILE_AND_EXECUTE		= 0x1301,
	MGL_LIST_BASE			= 0x0B32,
	MGL_LIST_INDEX			= 0x0B33,
	MGL_LIST_MODE			= 0x0B30,

	/* Depth buffer */
	MGL_NEVER			= 0x0200,
	MGL_LESS				= 0x0201,
	MGL_GEQUAL			= 0x0206,
	MGL_LEQUAL			= 0x0203,
	MGL_GREATER			= 0x0204,
	MGL_NOTEQUAL			= 0x0205,
	MGL_EQUAL			= 0x0202,
	MGL_ALWAYS			= 0x0207,
	MGL_DEPTH_TEST			= 0x0B71,
	MGL_DEPTH_BITS			= 0x0D56,
	MGL_DEPTH_CLEAR_VALUE		= 0x0B73,
	MGL_DEPTH_FUNC			= 0x0B74,
	MGL_DEPTH_RANGE			= 0x0B70,
	MGL_DEPTH_WRITEMASK		= 0x0B72,
	MGL_DEPTH_COMPONENT		= 0x1902,

	/* Lighting */
	MGL_LIGHTING			= 0x0B50,
	MGL_LIGHT0			= 0x4000,
	MGL_LIGHT1			= 0x4001,
	MGL_LIGHT2			= 0x4002,
	MGL_LIGHT3			= 0x4003,
	MGL_LIGHT4			= 0x4004,
	MGL_LIGHT5			= 0x4005,
	MGL_LIGHT6			= 0x4006,
	MGL_LIGHT7			= 0x4007,
	MGL_SPOT_EXPONENT		= 0x1205,
	MGL_SPOT_CUTOFF			= 0x1206,
	MGL_CONSTANT_ATTENUATION		= 0x1207,
	MGL_LINEAR_ATTENUATION		= 0x1208,
	MGL_QUADRATIC_ATTENUATION	= 0x1209,
	MGL_AMBIENT			= 0x1200,
	MGL_DIFFUSE			= 0x1201,
	MGL_SPECULAR			= 0x1202,
	MGL_SHININESS			= 0x1601,
	MGL_EMISSION			= 0x1600,
	MGL_POSITION			= 0x1203,
	MGL_SPOT_DIRECTION		= 0x1204,
	MGL_AMBIENT_AND_DIFFUSE		= 0x1602,
	MGL_COLOR_INDEXES		= 0x1603,
	MGL_LIGHT_MODEL_TWO_SIDE		= 0x0B52,
	MGL_LIGHT_MODEL_LOCAL_VIEWER	= 0x0B51,
	MGL_LIGHT_MODEL_AMBIENT		= 0x0B53,
	MGL_FRONT_AND_BACK		= 0x0408,
	MGL_SHADE_MODEL			= 0x0B54,
	MGL_FLAT				= 0x1D00,
	MGL_SMOOTH			= 0x1D01,
	MGL_COLOR_MATERIAL		= 0x0B57,
	MGL_COLOR_MATERIAL_FACE		= 0x0B55,
	MGL_COLOR_MATERIAL_PARAMETER	= 0x0B56,
	MGL_NORMALIZE			= 0x0BA1,

	/* User clipping planes */
	MGL_CLIP_PLANE0			= 0x3000,
	MGL_CLIP_PLANE1			= 0x3001,
	MGL_CLIP_PLANE2			= 0x3002,
	MGL_CLIP_PLANE3			= 0x3003,
	MGL_CLIP_PLANE4			= 0x3004,
	MGL_CLIP_PLANE5			= 0x3005,

	/* Accumulation buffer */
	MGL_ACCUM_RED_BITS		= 0x0D58,
	MGL_ACCUM_GREEN_BITS		= 0x0D59,
	MGL_ACCUM_BLUE_BITS		= 0x0D5A,
	MGL_ACCUM_ALPHA_BITS		= 0x0D5B,
	MGL_ACCUM_CLEAR_VALUE		= 0x0B80,
	MGL_ACCUM			= 0x0100,
	MGL_ADD				= 0x0104,
	MGL_LOAD				= 0x0101,
	MGL_MULT				= 0x0103,
	MGL_RETURN			= 0x0102,

	/* Alpha testing */
	MGL_ALPHA_TEST			= 0x0BC0,
	MGL_ALPHA_TEST_REF		= 0x0BC2,
	MGL_ALPHA_TEST_FUNC		= 0x0BC1,

	/* Blending */
	MGL_BLEND			= 0x0BE2,
	MGL_BLEND_SRC			= 0x0BE1,
	MGL_BLEND_DST			= 0x0BE0,
	MGL_ZERO				= 0,
	MGL_ONE				= 1,
	MGL_SRC_COLOR			= 0x0300,
	MGL_ONE_MINUS_SRC_COLOR		= 0x0301,
	MGL_DST_COLOR			= 0x0306,
	MGL_ONE_MINUS_DST_COLOR		= 0x0307,
	MGL_SRC_ALPHA			= 0x0302,
	MGL_ONE_MINUS_SRC_ALPHA		= 0x0303,
	MGL_DST_ALPHA			= 0x0304,
	MGL_ONE_MINUS_DST_ALPHA		= 0x0305,
	MGL_SRC_ALPHA_SATURATE		= 0x0308,
	MGL_CONSTANT_COLOR		= 0x8001,
	MGL_ONE_MINUS_CONSTANT_COLOR	= 0x8002,
	MGL_CONSTANT_ALPHA		= 0x8003,
	MGL_ONE_MINUS_CONSTANT_ALPHA	= 0x8004,

	/* Render Mode */
	MGL_FEEDBACK			= 0x1C01,
	MGL_RENDER			= 0x1C00,
	MGL_SELECT			= 0x1C02,

	/* Feedback */
	MGL_2D				= 0x0600,
	MGL_3D				= 0x0601,
	MGL_3D_COLOR			= 0x0602,
	MGL_3D_COLOR_TEXTURE		= 0x0603,
	MGL_4D_COLOR_TEXTURE		= 0x0604,
	MGL_POINT_TOKEN			= 0x0701,
	MGL_LINE_TOKEN			= 0x0702,
	MGL_LINE_RESET_TOKEN		= 0x0707,
	MGL_POLYGON_TOKEN		= 0x0703,
	MGL_BITMAP_TOKEN			= 0x0704,
	MGL_DRAW_PIXEL_TOKEN		= 0x0705,
	MGL_COPY_PIXEL_TOKEN		= 0x0706,
	MGL_PASS_THROUGH_TOKEN		= 0x0700,
	MGL_FEEDBACK_BUFFER_POINTER	= 0x0DF0,
	MGL_FEEDBACK_BUFFER_SIZE		= 0x0DF1,
	MGL_FEEDBACK_BUFFER_TYPE		= 0x0DF2,

	/* Selection */
	MGL_SELECTION_BUFFER_POINTER	= 0x0DF3,
	MGL_SELECTION_BUFFER_SIZE	= 0x0DF4,

	/* Fog */
	MGL_FOG				= 0x0B60,
	MGL_FOG_MODE			= 0x0B65,
	MGL_FOG_DENSITY			= 0x0B62,
	MGL_FOG_COLOR			= 0x0B66,
	MGL_FOG_INDEX			= 0x0B61,
	MGL_FOG_START			= 0x0B63,
	MGL_FOG_END			= 0x0B64,
	MGL_LINEAR			= 0x2601,
	MGL_EXP				= 0x0800,
	MGL_EXP2				= 0x0801,

	/* Logic Ops */
	MGL_LOGIC_OP			= 0x0BF1,
	MGL_INDEX_LOGIC_OP		= 0x0BF1,
	MGL_COLOR_LOGIC_OP		= 0x0BF2,
	MGL_LOGIC_OP_MODE		= 0x0BF0,
	MGL_CLEAR			= 0x1500,
	MGL_SET				= 0x150F,
	MGL_COPY				= 0x1503,
	MGL_COPY_INVERTED		= 0x150C,
	MGL_NOOP				= 0x1505,
	MGL_INVERT			= 0x150A,
	MGL_AND				= 0x1501,
	MGL_NAND				= 0x150E,
	MGL_OR				= 0x1507,
	MGL_NOR				= 0x1508,
	MGL_XOR				= 0x1506,
	MGL_EQUIV			= 0x1509,
	MGL_AND_REVERSE			= 0x1502,
	MGL_AND_INVERTED			= 0x1504,
	MGL_OR_REVERSE			= 0x150B,
	MGL_OR_INVERTED			= 0x150D,

	/* Stencil */
	MGL_STENCIL_TEST			= 0x0B90,
	MGL_STENCIL_WRITEMASK		= 0x0B98,
	MGL_STENCIL_BITS			= 0x0D57,
	MGL_STENCIL_FUNC			= 0x0B92,
	MGL_STENCIL_VALUE_MASK		= 0x0B93,
	MGL_STENCIL_REF			= 0x0B97,
	MGL_STENCIL_FAIL			= 0x0B94,
	MGL_STENCIL_PASS_DEPTH_PASS	= 0x0B96,
	MGL_STENCIL_PASS_DEPTH_FAIL	= 0x0B95,
	MGL_STENCIL_CLEAR_VALUE		= 0x0B91,
	MGL_STENCIL_INDEX		= 0x1901,
	MGL_KEEP				= 0x1E00,
	MGL_REPLACE			= 0x1E01,
	MGL_INCR				= 0x1E02,
	MGL_DECR				= 0x1E03,

	/* Buffers, Pixel Drawing/Reading */
	MGL_NONE				= 0,
	MGL_LEFT				= 0x0406,
	MGL_RIGHT			= 0x0407,
	/*MGL_FRONT			= 0x0404, */
	/*MGL_BACK			= 0x0405, */
	/*MGL_FRONT_AND_BACK		= 0x0408, */
	MGL_FRONT_LEFT			= 0x0400,
	MGL_FRONT_RIGHT			= 0x0401,
	MGL_BACK_LEFT			= 0x0402,
	MGL_BACK_RIGHT			= 0x0403,
	MGL_AUX0				= 0x0409,
	MGL_AUX1				= 0x040A,
	MGL_AUX2				= 0x040B,
	MGL_AUX3				= 0x040C,
	MGL_COLOR_INDEX			= 0x1900,
	MGL_RED				= 0x1903,
	MGL_GREEN			= 0x1904,
	MGL_BLUE				= 0x1905,
	MGL_ALPHA			= 0x1906,
	MGL_LUMINANCE			= 0x1909,
	MGL_LUMINANCE_ALPHA		= 0x190A,
	MGL_ALPHA_BITS			= 0x0D55,
	MGL_RED_BITS			= 0x0D52,
	MGL_GREEN_BITS			= 0x0D53,
	MGL_BLUE_BITS			= 0x0D54,
	MGL_INDEX_BITS			= 0x0D51,
	MGL_SUBPIXEL_BITS		= 0x0D50,
	MGL_AUX_BUFFERS			= 0x0C00,
	MGL_READ_BUFFER			= 0x0C02,
	MGL_DRAW_BUFFER			= 0x0C01,
	MGL_DOUBLEBUFFER			= 0x0C32,
	MGL_STEREO			= 0x0C33,
	MGL_BITMAP			= 0x1A00,
	MGL_COLOR			= 0x1800,
	MGL_DEPTH			= 0x1801,
	MGL_STENCIL			= 0x1802,
	MGL_DITHER			= 0x0BD0,
	MGL_RGB				= 0x1907,
	MGL_RGBA				= 0x1908,

	/* Implementation limits */
	MGL_MAX_LIST_NESTING		= 0x0B31,
	MGL_MAX_ATTRIB_STACK_DEPTH	= 0x0D35,
	MGL_MAX_MODELVIEW_STACK_DEPTH	= 0x0D36,
	MGL_MAX_NAME_STACK_DEPTH		= 0x0D37,
	MGL_MAX_PROJECTION_STACK_DEPTH	= 0x0D38,
	MGL_MAX_TEXTURE_STACK_DEPTH	= 0x0D39,
	MGL_MAX_EVAL_ORDER		= 0x0D30,
	MGL_MAX_LIGHTS			= 0x0D31,
	MGL_MAX_CLIP_PLANES		= 0x0D32,
	MGL_MAX_TEXTURE_SIZE		= 0x0D33,
	MGL_MAX_PIXEL_MAP_TABLE		= 0x0D34,
	MGL_MAX_VIEWPORT_DIMS		= 0x0D3A,
	MGL_MAX_CLIENT_ATTRIB_STACK_DEPTH= 0x0D3B,

	/* Gets */
	MGL_ATTRIB_STACK_DEPTH		= 0x0BB0,
	MGL_CLIENT_ATTRIB_STACK_DEPTH	= 0x0BB1,
	MGL_COLOR_CLEAR_VALUE		= 0x0C22,
	MGL_COLOR_WRITEMASK		= 0x0C23,
	MGL_CURRENT_INDEX		= 0x0B01,
	MGL_CURRENT_COLOR		= 0x0B00,
	MGL_CURRENT_NORMAL		= 0x0B02,
	MGL_CURRENT_RASTER_COLOR		= 0x0B04,
	MGL_CURRENT_RASTER_DISTANCE	= 0x0B09,
	MGL_CURRENT_RASTER_INDEX		= 0x0B05,
	MGL_CURRENT_RASTER_POSITION	= 0x0B07,
	MGL_CURRENT_RASTER_TEXTURE_COORDS = 0x0B06,
	MGL_CURRENT_RASTER_POSITION_VALID = 0x0B08,
	MGL_CURRENT_TEXTURE_COORDS	= 0x0B03,
	MGL_INDEX_CLEAR_VALUE		= 0x0C20,
	MGL_INDEX_MODE			= 0x0C30,
	MGL_INDEX_WRITEMASK		= 0x0C21,
	MGL_MODELVIEW_MATRIX		= 0x0BA6,
	MGL_MODELVIEW_STACK_DEPTH	= 0x0BA3,
	MGL_NAME_STACK_DEPTH		= 0x0D70,
	MGL_PROJECTION_MATRIX		= 0x0BA7,
	MGL_PROJECTION_STACK_DEPTH	= 0x0BA4,
	MGL_RENDER_MODE			= 0x0C40,
	MGL_RGBA_MODE			= 0x0C31,
	MGL_TEXTURE_MATRIX		= 0x0BA8,
	MGL_TEXTURE_STACK_DEPTH		= 0x0BA5,
	MGL_VIEWPORT			= 0x0BA2,

	/* Evaluators */
	MGL_AUTO_NORMAL			= 0x0D80,
	MGL_MAP1_COLOR_4			= 0x0D90,
	MGL_MAP1_GRID_DOMAIN		= 0x0DD0,
	MGL_MAP1_GRID_SEGMENTS		= 0x0DD1,
	MGL_MAP1_INDEX			= 0x0D91,
	MGL_MAP1_NORMAL			= 0x0D92,
	MGL_MAP1_TEXTURE_COORD_1		= 0x0D93,
	MGL_MAP1_TEXTURE_COORD_2		= 0x0D94,
	MGL_MAP1_TEXTURE_COORD_3		= 0x0D95,
	MGL_MAP1_TEXTURE_COORD_4		= 0x0D96,
	MGL_MAP1_VERTEX_3		= 0x0D97,
	MGL_MAP1_VERTEX_4		= 0x0D98,
	MGL_MAP2_COLOR_4			= 0x0DB0,
	MGL_MAP2_GRID_DOMAIN		= 0x0DD2,
	MGL_MAP2_GRID_SEGMENTS		= 0x0DD3,
	MGL_MAP2_INDEX			= 0x0DB1,
	MGL_MAP2_NORMAL			= 0x0DB2,
	MGL_MAP2_TEXTURE_COORD_1		= 0x0DB3,
	MGL_MAP2_TEXTURE_COORD_2		= 0x0DB4,
	MGL_MAP2_TEXTURE_COORD_3		= 0x0DB5,
	MGL_MAP2_TEXTURE_COORD_4		= 0x0DB6,
	MGL_MAP2_VERTEX_3		= 0x0DB7,
	MGL_MAP2_VERTEX_4		= 0x0DB8,
	MGL_COEFF			= 0x0A00,
	MGL_DOMAIN			= 0x0A02,
	MGL_ORDER			= 0x0A01,

	/* Hints */
	MGL_FOG_HINT			= 0x0C54,
	MGL_LINE_SMOOTH_HINT		= 0x0C52,
	MGL_PERSPECTIVE_CORRECTION_HINT	= 0x0C50,
	MGL_POINT_SMOOTH_HINT		= 0x0C51,
	MGL_POLYGON_SMOOTH_HINT		= 0x0C53,
	MGL_DONT_CARE			= 0x1100,
	MGL_FASTEST			= 0x1101,
	MGL_NICEST			= 0x1102,

	/* Scissor box */
	MGL_SCISSOR_TEST			= 0x0C11,
	MGL_SCISSOR_BOX			= 0x0C10,

	/* Pixel Mode / Transfer */
	MGL_MAP_COLOR			= 0x0D10,
	MGL_MAP_STENCIL			= 0x0D11,
	MGL_INDEX_SHIFT			= 0x0D12,
	MGL_INDEX_OFFSET			= 0x0D13,
	MGL_RED_SCALE			= 0x0D14,
	MGL_RED_BIAS			= 0x0D15,
	MGL_GREEN_SCALE			= 0x0D18,
	MGL_GREEN_BIAS			= 0x0D19,
	MGL_BLUE_SCALE			= 0x0D1A,
	MGL_BLUE_BIAS			= 0x0D1B,
	MGL_ALPHA_SCALE			= 0x0D1C,
	MGL_ALPHA_BIAS			= 0x0D1D,
	MGL_DEPTH_SCALE			= 0x0D1E,
	MGL_DEPTH_BIAS			= 0x0D1F,
	MGL_PIXEL_MAP_S_TO_S_SIZE	= 0x0CB1,
	MGL_PIXEL_MAP_I_TO_I_SIZE	= 0x0CB0,
	MGL_PIXEL_MAP_I_TO_R_SIZE	= 0x0CB2,
	MGL_PIXEL_MAP_I_TO_G_SIZE	= 0x0CB3,
	MGL_PIXEL_MAP_I_TO_B_SIZE	= 0x0CB4,
	MGL_PIXEL_MAP_I_TO_A_SIZE	= 0x0CB5,
	MGL_PIXEL_MAP_R_TO_R_SIZE	= 0x0CB6,
	MGL_PIXEL_MAP_G_TO_G_SIZE	= 0x0CB7,
	MGL_PIXEL_MAP_B_TO_B_SIZE	= 0x0CB8,
	MGL_PIXEL_MAP_A_TO_A_SIZE	= 0x0CB9,
	MGL_PIXEL_MAP_S_TO_S		= 0x0C71,
	MGL_PIXEL_MAP_I_TO_I		= 0x0C70,
	MGL_PIXEL_MAP_I_TO_R		= 0x0C72,
	MGL_PIXEL_MAP_I_TO_G		= 0x0C73,
	MGL_PIXEL_MAP_I_TO_B		= 0x0C74,
	MGL_PIXEL_MAP_I_TO_A		= 0x0C75,
	MGL_PIXEL_MAP_R_TO_R		= 0x0C76,
	MGL_PIXEL_MAP_G_TO_G		= 0x0C77,
	MGL_PIXEL_MAP_B_TO_B		= 0x0C78,
	MGL_PIXEL_MAP_A_TO_A		= 0x0C79,
	MGL_PACK_ALIGNMENT		= 0x0D05,
	MGL_PACK_LSB_FIRST		= 0x0D01,
	MGL_PACK_ROW_LENGTH		= 0x0D02,
	MGL_PACK_SKIP_PIXELS		= 0x0D04,
	MGL_PACK_SKIP_ROWS		= 0x0D03,
	MGL_PACK_SWAP_BYTES		= 0x0D00,
	MGL_UNPACK_ALIGNMENT		= 0x0CF5,
	MGL_UNPACK_LSB_FIRST		= 0x0CF1,
	MGL_UNPACK_ROW_LENGTH		= 0x0CF2,
	MGL_UNPACK_SKIP_PIXELS		= 0x0CF4,
	MGL_UNPACK_SKIP_ROWS		= 0x0CF3,
	MGL_UNPACK_SWAP_BYTES		= 0x0CF0,
	MGL_ZOOM_X			= 0x0D16,
	MGL_ZOOM_Y			= 0x0D17,

	/* Texture mapping */
	MGL_TEXTURE_ENV			= 0x2300,
	MGL_TEXTURE_ENV_MODE		= 0x2200,
	MGL_TEXTURE_1D			= 0x0DE0,
	MGL_TEXTURE_2D			= 0x0DE1,
	MGL_TEXTURE_WRAP_S		= 0x2802,
	MGL_TEXTURE_WRAP_T		= 0x2803,
	MGL_TEXTURE_MAG_FILTER		= 0x2800,
	MGL_TEXTURE_MIN_FILTER		= 0x2801,
	MGL_TEXTURE_ENV_COLOR		= 0x2201,
	MGL_TEXTURE_GEN_S		= 0x0C60,
	MGL_TEXTURE_GEN_T		= 0x0C61,
	MGL_TEXTURE_GEN_MODE		= 0x2500,
	MGL_TEXTURE_BORDER_COLOR		= 0x1004,
	MGL_TEXTURE_WIDTH		= 0x1000,
	MGL_TEXTURE_HEIGHT		= 0x1001,
	MGL_TEXTURE_BORDER		= 0x1005,
	MGL_TEXTURE_COMPONENTS		= 0x1003,
	MGL_TEXTURE_RED_SIZE		= 0x805C,
	MGL_TEXTURE_GREEN_SIZE		= 0x805D,
	MGL_TEXTURE_BLUE_SIZE		= 0x805E,
	MGL_TEXTURE_ALPHA_SIZE		= 0x805F,
	MGL_TEXTURE_LUMINANCE_SIZE	= 0x8060,
	MGL_TEXTURE_INTENSITY_SIZE	= 0x8061,
	MGL_NEAREST_MIPMAP_NEAREST	= 0x2700,
	MGL_NEAREST_MIPMAP_LINEAR	= 0x2702,
	MGL_LINEAR_MIPMAP_NEAREST	= 0x2701,
	MGL_LINEAR_MIPMAP_LINEAR		= 0x2703,
	MGL_OBJECT_LINEAR		= 0x2401,
	MGL_OBJECT_PLANE			= 0x2501,
	MGL_EYE_LINEAR			= 0x2400,
	MGL_EYE_PLANE			= 0x2502,
	MGL_SPHERE_MAP			= 0x2402,
	MGL_DECAL			= 0x2101,
	MGL_MODULATE			= 0x2100,
	MGL_NEAREST			= 0x2600,
	MGL_REPEAT			= 0x2901,
	MGL_CLAMP			= 0x2900,
	MGL_S				= 0x2000,
	MGL_T				= 0x2001,
	MGL_R				= 0x2002,
	MGL_Q				= 0x2003,
	MGL_TEXTURE_GEN_R		= 0x0C62,
	MGL_TEXTURE_GEN_Q		= 0x0C63,

	/* GL 1.1 texturing */
	MGL_PROXY_TEXTURE_1D		= 0x8063,
	MGL_PROXY_TEXTURE_2D		= 0x8064,
	MGL_TEXTURE_PRIORITY		= 0x8066,
	MGL_TEXTURE_RESIDENT		= 0x8067,
	MGL_TEXTURE_BINDING_1D		= 0x8068,
	MGL_TEXTURE_BINDING_2D		= 0x8069,
	MGL_TEXTURE_INTERNAL_FORMAT	= 0x1003,

	/* GL 1.2 texturing */
	MGL_PACK_SKIP_IMAGES		= 0x806B,
	MGL_PACK_IMAGE_HEIGHT		= 0x806C,
	MGL_UNPACK_SKIP_IMAGES		= 0x806D,
	MGL_UNPACK_IMAGE_HEIGHT		= 0x806E,
	MGL_TEXTURE_3D			= 0x806F,
	MGL_PROXY_TEXTURE_3D		= 0x8070,
	MGL_TEXTURE_DEPTH		= 0x8071,
	MGL_TEXTURE_WRAP_R		= 0x8072,
	MGL_MAX_3D_TEXTURE_SIZE		= 0x8073,
	MGL_TEXTURE_BINDING_3D		= 0x806A,

	/* Internal texture formats (GL 1.1) */
	MGL_ALPHA4			= 0x803B,
	MGL_ALPHA8			= 0x803C,
	MGL_ALPHA12			= 0x803D,
	MGL_ALPHA16			= 0x803E,
	MGL_LUMINANCE4			= 0x803F,
	MGL_LUMINANCE8			= 0x8040,
	MGL_LUMINANCE12			= 0x8041,
	MGL_LUMINANCE16			= 0x8042,
	MGL_LUMINANCE4_ALPHA4		= 0x8043,
	MGL_LUMINANCE6_ALPHA2		= 0x8044,
	MGL_LUMINANCE8_ALPHA8		= 0x8045,
	MGL_LUMINANCE12_ALPHA4		= 0x8046,
	MGL_LUMINANCE12_ALPHA12		= 0x8047,
	MGL_LUMINANCE16_ALPHA16		= 0x8048,
	MGL_INTENSITY			= 0x8049,
	MGL_INTENSITY4			= 0x804A,
	MGL_INTENSITY8			= 0x804B,
	MGL_INTENSITY12			= 0x804C,
	MGL_INTENSITY16			= 0x804D,
	MGL_R3_G3_B2			= 0x2A10,
	MGL_RGB4				= 0x804F,
	MGL_RGB5				= 0x8050,
	MGL_RGB8				= 0x8051,
	MGL_RGB10			= 0x8052,
	MGL_RGB12			= 0x8053,
	MGL_RGB16			= 0x8054,
	MGL_RGBA2			= 0x8055,
	MGL_RGBA4			= 0x8056,
	MGL_RGB5_A1			= 0x8057,
	MGL_RGBA8			= 0x8058,
	MGL_RGB10_A2			= 0x8059,
	MGL_RGBA12			= 0x805A,
	MGL_RGBA16			= 0x805B,

	/* Utility */
	MGL_VENDOR			= 0x1F00,
	MGL_RENDERER			= 0x1F01,
	MGL_VERSION			= 0x1F02,
	MGL_EXTENSIONS			= 0x1F03,

	/* Errors */
	MGL_INVALID_VALUE		= 0x0501,
	MGL_INVALID_ENUM			= 0x0500,
	MGL_INVALID_OPERATION		= 0x0502,
	MGL_STACK_OVERFLOW		= 0x0503,
	MGL_STACK_UNDERFLOW		= 0x0504,
	MGL_OUT_OF_MEMORY		= 0x0505,

	/*
	 * Extensions
	 */

	/* MGL_EXT_blend_minmax and MGL_EXT_blend_color */
	MGL_CONSTANT_COLOR_EXT			= 0x8001,
	MGL_ONE_MINUS_CONSTANT_COLOR_EXT		= 0x8002,
	MGL_CONSTANT_ALPHA_EXT			= 0x8003,
	MGL_ONE_MINUS_CONSTANT_ALPHA_EXT		= 0x8004,
	MGL_BLEND_EQUATION_EXT			= 0x8009,
	MGL_MIN_EXT				= 0x8007,
	MGL_MAX_EXT				= 0x8008,
	MGL_FUNC_ADD_EXT				= 0x8006,
	MGL_FUNC_SUBTRACT_EXT			= 0x800A,
	MGL_FUNC_REVERSE_SUBTRACT_EXT		= 0x800B,
	MGL_BLEND_COLOR_EXT			= 0x8005,

	/* MGL_EXT_polygon_offset */
	MGL_POLYGON_OFFSET_EXT			= 0x8037,
	MGL_POLYGON_OFFSET_FACTOR_EXT		= 0x8038,
	MGL_POLYGON_OFFSET_BIAS_EXT		= 0x8039,

	/* MGL_EXT_vertex_array */
	MGL_VERTEX_ARRAY_EXT			= 0x8074,
	MGL_NORMAL_ARRAY_EXT			= 0x8075,
	MGL_COLOR_ARRAY_EXT			= 0x8076,
	MGL_INDEX_ARRAY_EXT			= 0x8077,
	MGL_TEXTURE_COORD_ARRAY_EXT		= 0x8078,
	MGL_EDGE_FLAG_ARRAY_EXT			= 0x8079,
	MGL_VERTEX_ARRAY_SIZE_EXT		= 0x807A,
	MGL_VERTEX_ARRAY_TYPE_EXT		= 0x807B,
	MGL_VERTEX_ARRAY_STRIDE_EXT		= 0x807C,
	MGL_VERTEX_ARRAY_COUNT_EXT		= 0x807D,
	MGL_NORMAL_ARRAY_TYPE_EXT		= 0x807E,
	MGL_NORMAL_ARRAY_STRIDE_EXT		= 0x807F,
	MGL_NORMAL_ARRAY_COUNT_EXT		= 0x8080,
	MGL_COLOR_ARRAY_SIZE_EXT			= 0x8081,
	MGL_COLOR_ARRAY_TYPE_EXT			= 0x8082,
	MGL_COLOR_ARRAY_STRIDE_EXT		= 0x8083,
	MGL_COLOR_ARRAY_COUNT_EXT		= 0x8084,
	MGL_INDEX_ARRAY_TYPE_EXT			= 0x8085,
	MGL_INDEX_ARRAY_STRIDE_EXT		= 0x8086,
	MGL_INDEX_ARRAY_COUNT_EXT		= 0x8087,
	MGL_TEXTURE_COORD_ARRAY_SIZE_EXT		= 0x8088,
	MGL_TEXTURE_COORD_ARRAY_TYPE_EXT		= 0x8089,
	MGL_TEXTURE_COORD_ARRAY_STRIDE_EXT	= 0x808A,
	MGL_TEXTURE_COORD_ARRAY_COUNT_EXT	= 0x808B,
	MGL_EDGE_FLAG_ARRAY_STRIDE_EXT		= 0x808C,
	MGL_EDGE_FLAG_ARRAY_COUNT_EXT		= 0x808D,
	MGL_VERTEX_ARRAY_POINTER_EXT		= 0x808E,
	MGL_NORMAL_ARRAY_POINTER_EXT		= 0x808F,
	MGL_COLOR_ARRAY_POINTER_EXT		= 0x8090,
	MGL_INDEX_ARRAY_POINTER_EXT		= 0x8091,
	MGL_TEXTURE_COORD_ARRAY_POINTER_EXT	= 0x8092,
	MGL_EDGE_FLAG_ARRAY_POINTER_EXT		= 0x8093,

	/* MGL_EXT_texture_object */
	MGL_TEXTURE_PRIORITY_EXT			= 0x8066,
	MGL_TEXTURE_RESIDENT_EXT			= 0x8067,
	MGL_TEXTURE_1D_BINDING_EXT		= 0x8068,
	MGL_TEXTURE_2D_BINDING_EXT		= 0x8069,

	/* MGL_EXT_texture3D */
	MGL_PACK_SKIP_IMAGES_EXT			= 0x806B,
	MGL_PACK_IMAGE_HEIGHT_EXT		= 0x806C,
	MGL_UNPACK_SKIP_IMAGES_EXT		= 0x806D,
	MGL_UNPACK_IMAGE_HEIGHT_EXT		= 0x806E,
	MGL_TEXTURE_3D_EXT			= 0x806F,
	MGL_PROXY_TEXTURE_3D_EXT			= 0x8070,
	MGL_TEXTURE_DEPTH_EXT			= 0x8071,
	MGL_TEXTURE_WRAP_R_EXT			= 0x8072,
	MGL_MAX_3D_TEXTURE_SIZE_EXT		= 0x8073,
	MGL_TEXTURE_3D_BINDING_EXT		= 0x806A,

	/* MGL_EXT_paletted_texture */
	MGL_TABLE_TOO_LARGE_EXT			= 0x8031,
	MGL_COLOR_TABLE_FORMAT_EXT		= 0x80D8,
	MGL_COLOR_TABLE_WIDTH_EXT		= 0x80D9,
	MGL_COLOR_TABLE_RED_SIZE_EXT		= 0x80DA,
	MGL_COLOR_TABLE_GREEN_SIZE_EXT		= 0x80DB,
	MGL_COLOR_TABLE_BLUE_SIZE_EXT		= 0x80DC,
	MGL_COLOR_TABLE_ALPHA_SIZE_EXT	 	= 0x80DD,
	MGL_COLOR_TABLE_LUMINANCE_SIZE_EXT	= 0x80DE,
	MGL_COLOR_TABLE_INTENSITY_SIZE_EXT	= 0x80DF,
	MGL_TEXTURE_INDEX_SIZE_EXT		= 0x80ED,
	MGL_COLOR_INDEX1_EXT			= 0x80E2,
	MGL_COLOR_INDEX2_EXT			= 0x80E3,
	MGL_COLOR_INDEX4_EXT			= 0x80E4,
	MGL_COLOR_INDEX8_EXT			= 0x80E5,
	MGL_COLOR_INDEX12_EXT			= 0x80E6,
	MGL_COLOR_INDEX16_EXT			= 0x80E7,

	/* MGL_EXT_shared_texture_palette */
	MGL_SHARED_TEXTURE_PALETTE_EXT		= 0x81FB,

	/* MGL_EXT_point_parameters */
	MGL_POINT_SIZE_MIN_EXT			= 0x8126,
	MGL_POINT_SIZE_MAX_EXT			= 0x8127,
	MGL_POINT_FADE_THRESHOLD_SIZE_EXT	= 0x8128,
	MGL_DISTANCE_ATTENUATION_EXT		= 0x8129,

	/* MGL_EXT_rescale_normal */
	MGL_RESCALE_NORMAL_EXT			= 0x803A,

	/* MGL_EXT_abgr */
	MGL_ABGR_EXT				= 0x8000,

	/* MGL_EXT_stencil_wrap */
	MGL_INCR_WRAP_EXT			= 0x8507,
	MGL_DECR_WRAP_EXT			= 0x8508,

	/* MGL_SGIS_texture_edge_clamp */
	MGL_CLAMP_TO_EDGE_SGIS			= 0x812F,

	/* MGL_INGR_blend_func_separate */
	MGL_BLEND_DST_RGB_INGR			= 0x80C8,
	MGL_BLEND_SRC_RGB_INGR			= 0x80C9,
	MGL_BLEND_DST_ALPHA_INGR			= 0x80CA,
	MGL_BLEND_SRC_ALPHA_INGR			= 0x80CB,

	/* OpenGL 1.2 */
	MGL_RESCALE_NORMAL			= 0x803A,
	MGL_CLAMP_TO_EDGE			= 0x812F,
	MGL_MAX_ELEMENTS_VERTICES		= 0x80E8,
	MGL_MAX_ELEMENTS_INDICES			= 0x80E9,
	MGL_BGR					= 0x80E0,
	MGL_BGRA					= 0x80E1,
	MGL_UNSIGNED_BYTE_3_3_2			= 0x8032,
	MGL_UNSIGNED_BYTE_2_3_3_REV		= 0x8362,
	MGL_UNSIGNED_SHORT_5_6_5			= 0x8363,
	MGL_UNSIGNED_SHORT_5_6_5_REV		= 0x8364,
	MGL_UNSIGNED_SHORT_4_4_4_4		= 0x8033,
	MGL_UNSIGNED_SHORT_4_4_4_4_REV		= 0x8365,
	MGL_UNSIGNED_SHORT_5_5_5_1		= 0x8034,
	MGL_UNSIGNED_SHORT_1_5_5_5_REV		= 0x8366,
	MGL_UNSIGNED_INT_8_8_8_8			= 0x8035,
	MGL_UNSIGNED_INT_8_8_8_8_REV		= 0x8367,
	MGL_UNSIGNED_INT_10_10_10_2		= 0x8036,
	MGL_UNSIGNED_INT_2_10_10_10_REV		= 0x8368,
	MGL_LIGHT_MODEL_COLOR_CONTROL		= 0x81F8,
	MGL_SINGLE_COLOR				= 0x81F9,
	MGL_SEPARATE_SPECULAR_COLOR		= 0x81FA,
	MGL_TEXTURE_MIN_LOD			= 0x813A,
	MGL_TEXTURE_MAX_LOD			= 0x813B,
	MGL_TEXTURE_BASE_LEVEL			= 0x813C,
	MGL_TEXTURE_MAX_LEVEL			= 0x813D,

	/* MGL_ARB_multitexture */
	MGL_TEXTURE0_ARB				= 0x84C0,
	MGL_TEXTURE1_ARB				= 0x84C1,
	MGL_TEXTURE2_ARB				= 0x84C2,
	MGL_TEXTURE3_ARB				= 0x84C3,
	MGL_TEXTURE4_ARB				= 0x84C4,
	MGL_TEXTURE5_ARB				= 0x84C5,
	MGL_TEXTURE6_ARB				= 0x84C6,
	MGL_TEXTURE7_ARB				= 0x84C7,
	MGL_TEXTURE8_ARB				= 0x84C8,
	MGL_TEXTURE9_ARB				= 0x84C9,
	MGL_TEXTURE10_ARB			= 0x84CA,
	MGL_TEXTURE11_ARB			= 0x84CB,
	MGL_TEXTURE12_ARB			= 0x84CC,
	MGL_TEXTURE13_ARB			= 0x84CD,
	MGL_TEXTURE14_ARB			= 0x84CE,
	MGL_TEXTURE15_ARB			= 0x84CF,
	MGL_TEXTURE16_ARB			= 0x84D0,
	MGL_TEXTURE17_ARB			= 0x84D1,
	MGL_TEXTURE18_ARB			= 0x84D2,
	MGL_TEXTURE19_ARB			= 0x84D3,
	MGL_TEXTURE20_ARB			= 0x84D4,
	MGL_TEXTURE21_ARB			= 0x84D5,
	MGL_TEXTURE22_ARB			= 0x84D6,
	MGL_TEXTURE23_ARB			= 0x84D7,
	MGL_TEXTURE24_ARB			= 0x84D8,
	MGL_TEXTURE25_ARB			= 0x84D9,
	MGL_TEXTURE26_ARB			= 0x84DA,
	MGL_TEXTURE27_ARB			= 0x84DB,
	MGL_TEXTURE28_ARB			= 0x84DC,
	MGL_TEXTURE29_ARB			= 0x84DD,
	MGL_TEXTURE30_ARB			= 0x84DE,
	MGL_TEXTURE31_ARB			= 0x84DF,
	MGL_ACTIVE_TEXTURE_ARB			= 0x84E0,
	MGL_CLIENT_ACTIVE_TEXTURE_ARB		= 0x84E1,
	MGL_MAX_TEXTURE_UNITS_ARB		= 0x84E2,

	/*
	 * OpenGL 1.2 imaging subset (NOT IMPLEMENTED BY MESA)
	 */
	/* MGL_EXT_color_table */
	MGL_COLOR_TABLE				= 0x80D0,
	MGL_POST_CONVOLUTION_COLOR_TABLE		= 0x80D1,
	MGL_POST_COLOR_MATRIX_COLOR_TABLE	= 0x80D2,
	MGL_PROXY_COLOR_TABLE			= 0x80D3,
	MGL_PROXY_POST_CONVOLUTION_COLOR_TABLE	= 0x80D4,
	MGL_PROXY_POST_COLOR_MATRIX_COLOR_TABLE	= 0x80D5,
	MGL_COLOR_TABLE_SCALE			= 0x80D6,
	MGL_COLOR_TABLE_BIAS			= 0x80D7,
	MGL_COLOR_TABLE_FORMAT			= 0x80D8,
	MGL_COLOR_TABLE_WIDTH			= 0x80D9,
	MGL_COLOR_TABLE_RED_SIZE			= 0x80DA,
	MGL_COLOR_TABLE_GREEN_SIZE		= 0x80DB,
	MGL_COLOR_TABLE_BLUE_SIZE		= 0x80DC,
	MGL_COLOR_TABLE_ALPHA_SIZE		= 0x80DD,
	MGL_COLOR_TABLE_LUMINANCE_SIZE		= 0x80DE,
	MGL_COLOR_TABLE_INTENSITY_SIZE		= 0x80DF,
	/* MGL_EXT_convolution and MGL_HP_convolution_border_modes */
	MGL_CONVOLUTION_1D			= 0x8010,
	MGL_CONVOLUTION_2D			= 0x8011,
	MGL_SEPARABLE_2D				= 0x8012,
	MGL_CONVOLUTION_BORDER_MODE		= 0x8013,
	MGL_CONVOLUTION_FILTER_SCALE		= 0x8014,
	MGL_CONVOLUTION_FILTER_BIAS		= 0x8015,
	MGL_REDUCE				= 0x8016,
	MGL_CONVOLUTION_FORMAT			= 0x8017,
	MGL_CONVOLUTION_WIDTH			= 0x8018,
	MGL_CONVOLUTION_HEIGHT			= 0x8019,
	MGL_MAX_CONVOLUTION_WIDTH		= 0x801A,
	MGL_MAX_CONVOLUTION_HEIGHT		= 0x801B,
	MGL_POST_CONVOLUTION_RED_SCALE		= 0x801C,
	MGL_POST_CONVOLUTION_GREEN_SCALE		= 0x801D,
	MGL_POST_CONVOLUTION_BLUE_SCALE		= 0x801E,
	MGL_POST_CONVOLUTION_ALPHA_SCALE		= 0x801F,
	MGL_POST_CONVOLUTION_RED_BIAS		= 0x8020,
	MGL_POST_CONVOLUTION_GREEN_BIAS		= 0x8021,
	MGL_POST_CONVOLUTION_BLUE_BIAS		= 0x8022,
	MGL_POST_CONVOLUTION_ALPHA_BIAS		= 0x8023,
	MGL_CONSTANT_BORDER			= 0x8151,
	MGL_REPLICATE_BORDER			= 0x8153,
	MGL_CONVOLUTION_BORDER_COLOR		= 0x8154,
	/* MGL_SGI_color_matrix */
	MGL_COLOR_MATRIX				= 0x80B1,
	MGL_COLOR_MATRIX_STACK_DEPTH		= 0x80B2,
	MGL_MAX_COLOR_MATRIX_STACK_DEPTH		= 0x80B3,
	MGL_POST_COLOR_MATRIX_RED_SCALE		= 0x80B4,
	MGL_POST_COLOR_MATRIX_GREEN_SCALE	= 0x80B5,
	MGL_POST_COLOR_MATRIX_BLUE_SCALE		= 0x80B6,
	MGL_POST_COLOR_MATRIX_ALPHA_SCALE	= 0x80B7,
	MGL_POST_COLOR_MATRIX_RED_BIAS		= 0x80B8,
	MGL_POST_COLOR_MATRIX_GREEN_BIAS		= 0x80B9,
	MGL_POST_COLOR_MATRIX_BLUE_BIAS		= 0x80BA,
	MGL_POST_COLOR_MATRIX_ALPHA_BIAS		= 0x80BB,
	/* MGL_EXT_histogram */
	MGL_HISTOGRAM				= 0x8024,
	MGL_PROXY_HISTOGRAM			= 0x8025,
	MGL_HISTOGRAM_WIDTH			= 0x8026,
	MGL_HISTOGRAM_FORMAT			= 0x8027,
	MGL_HISTOGRAM_RED_SIZE			= 0x8028,
	MGL_HISTOGRAM_GREEN_SIZE			= 0x8029,
	MGL_HISTOGRAM_BLUE_SIZE			= 0x802A,
	MGL_HISTOGRAM_ALPHA_SIZE			= 0x802B,
	MGL_HISTOGRAM_LUMINANCE_SIZE		= 0x802C,
	MGL_HISTOGRAM_SINK			= 0x802D,
	MGL_MINMAX				= 0x802E,
	MGL_MINMAX_FORMAT			= 0x802F,
	MGL_MINMAX_SINK				= 0x8030,
	MGL_TABLE_TOO_LARGE			= 0x8031,
	/* MGL_EXT_blend_color, MGL_EXT_blend_minmax */
	MGL_BLEND_EQUATION			= 0x8009,
	MGL_MIN					= 0x8007,
	MGL_MAX					= 0x8008,
	MGL_FUNC_ADD				= 0x8006,
	MGL_FUNC_SUBTRACT			= 0x800A,
	MGL_FUNC_REVERSE_SUBTRACT		= 0x800B,
	MGL_BLEND_COLOR				= 0x8005,

	/* MGL_NV_texgen_reflection (nVidia) */
	MGL_NORMAL_MAP_NV			= 0x8511,
	MGL_REFLECTION_MAP_NV			= 0x8512,

	/* MGL_PGI_misc_hints */
	MGL_PREFER_DOUBLEBUFFER_HINT_PGI		= 107000,
	MGL_STRICT_DEPTHFUNC_HINT_PGI		= 107030,
	MGL_STRICT_LIGHTING_HINT_PGI		= 107031,
	MGL_STRICT_SCISSOR_HINT_PGI		= 107032,
	MGL_FULL_STIPPLE_HINT_PGI		= 107033,
	MGL_NATIVE_GRAPHICS_BEGIN_HINT_PGI	= 107011,
	MGL_NATIVE_GRAPHICS_END_HINT_PGI		= 107012,
	MGL_CONSERVE_MEMORY_HINT_PGI		= 107005,
	MGL_RECLAIM_MEMORY_HINT_PGI		= 107006,
	MGL_ALWAYS_FAST_HINT_PGI			= 107020,
	MGL_ALWAYS_SOFT_HINT_PGI			= 107021,
	MGL_ALLOW_DRAW_OBJ_HINT_PGI		= 107022,
	MGL_ALLOW_DRAW_WIN_HINT_PGI		= 107023,
	MGL_ALLOW_DRAW_FRG_HINT_PGI		= 107024,
	MGL_ALLOW_DRAW_SPN_HINT_PGI		= 107024,
	MGL_ALLOW_DRAW_MEM_HINT_PGI		= 107025,
	MGL_CLIP_NEAR_HINT_PGI			= 107040,
	MGL_CLIP_FAR_HINT_PGI			= 107041,
	MGL_WIDE_LINE_HINT_PGI		  	= 107042,
	MGL_BACK_NORMALS_HINT_PGI		= 107043,
	MGL_NATIVE_GRAPHICS_HANDLE_PGI		= 107010,

	/* MGL_EXT_compiled_vertex_array */
	MGL_ARRAY_ELEMENT_LOCK_FIRST_SGI		= 0x81A8,
	MGL_ARRAY_ELEMENT_LOCK_COUNT_SGI		= 0x81A9,

	/* MGL_EXT_clip_volume_hint */
	MGL_CLIP_VOLUME_CLIPPING_HINT_EXT	= 0x80F0

} MGLenum;

enum {
        MGL_CURRENT_BIT          = 0x00000001,
        MGL_POINT_BIT            = 0x00000002,
        MGL_LINE_BIT             = 0x00000004,
        MGL_POLYGON_BIT          = 0x00000008,
        MGL_POLYGON_STIPPLE_BIT  = 0x00000010,
        MGL_PIXEL_MODE_BIT       = 0x00000020,
        MGL_LIGHTING_BIT         = 0x00000040,
        MGL_FOG_BIT              = 0x00000080,
        MGL_DEPTH_BUFFER_BIT     = 0x00000100,
        MGL_ACCUM_BUFFER_BIT     = 0x00000200,
        MGL_STENCIL_BUFFER_BIT   = 0x00000400,
        MGL_VIEWPORT_BIT         = 0x00000800,
        MGL_TRANSFORM_BIT        = 0x00001000,
        MGL_ENABLE_BIT           = 0x00002000,
        MGL_COLOR_BUFFER_BIT     = 0x00004000,
        MGL_HINT_BIT             = 0x00008000,
        MGL_EVAL_BIT             = 0x00010000,
        MGL_LIST_BIT             = 0x00020000,
        MGL_TEXTURE_BIT          = 0x00040000,
        MGL_SCISSOR_BIT          = 0x00080000,
        MGL_ALL_ATTRIB_BITS      = 0x000FFFFF
};      

#endif
