//   Matrix class for CS4731: COMPUTER GRAPHICS
//   Copyright (C) 1999,2000  Mark R. Stevens
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the Free Software
//   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#ifndef MATRIX
#define MATRIX

// forward declaration
class Vertex;

// Matrix class definition
class Matrix {

 public:

  // constructors
  Matrix ();

  // access data
  float   operator () (int row, int col) const;
  float & operator () (int row, int col);
  float   operator () (int i) const;
  float & operator () (int i);

  // operators
  friend Matrix operator * (const Matrix &m1, const Matrix &m2);
  friend Vertex operator * (const Matrix &m1, const Vertex &v2);

  // construct matrices
  static Matrix translate(float x, float y, float z);

 private:

  // local data
  float data[16];

};

#endif





