//   Vertex class for CS4731: COMPUTER GRAPHICS
//   Copyright (C) 1999,2000  Mark R. Stevens
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the Free Software
//   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#ifndef VERTEX
#define VERTEX

// Vertex class definition
class Vertex {

 public:

  // constructors
//  Vertex (float x = 0.0, 
//	  float y = 0.0, 
//	  float z = 0.0, 
//	  float w = 1.0);
  
  Vertex (float x, 
	  float y, 
	  float z, 
	  float w);

  float   x () const;
  float   y () const;
  float   z () const;
  float   w () const;
  float & x ();
  float & y ();
  float & z ();
  float & w ();

  // convert to homogenous coordinates
  Vertex homogenous() const;

 private:

  // local data
  float p[4];


};

#endif





